<?php
/*
 * This file is part of the Easy-CMS project.
 * Easy-CMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Easy-CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Easy-CMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/**
 * @package    Core
 * @subpackage ACL
 */
 
/**
 */
include_once('config.inc.php');
include_once('classes/core/DBAccessorObject.class.php');
include_once('classes/core/DBManipulator.class.php');
include_once('classes/core/ACL/ACL.class.php');


/**
 * Manage the instances of ACL objects.
 *
 * @access     public
 * @author	   Cedric Francoys <cedric.francoys@easy-cms.org>
 * @version	   ACLManager.class, Sat Feb 12 16:09:53 CET 2005
 * @package	   Core
 * @subpackage ACL
 */
class ACLManager extends DBAccessorObject {
	/**
	 * Array of ACL objects
	 *
	 * @access private
	 * @var    array
	 */
  var $ACLTable;
	
	/**#@+
	 * @access public 
	 */
	/**
	 * Class Constructor
	 *
	 * @param object instance of DBManipulator<i>Driver</i>
	 */
  function ACLManager(&$dbManipulator) {
		parent::DBAccessorObject($dbManipulator);
		$this->ACLTable = array();
  }
	

	/**
	 * Loads ACL
	 *
	 * @param  mixed $id integer or indexed array
	 * @return boolean
	 */
	function loadACL($id, $loadList=true) {
		$db = &$this->getDBManipulator();
		
		$aclList = (array)$id;
		
		foreach ($aclList as $aclId) {
			$this->ACLTable[$aclId] = &new ACL($aclId);
		}
		
		$csv  = implode(',', $aclList);
		$cond = '`acl_id` in ('.$csv.')';
		
		if ($res = $db->getRecord(ACL_GROUPS_TABLE, -1, $cond)) {
			while ($record = $db->fetchArray($res)) {
				$aclId = (int)$record['acl_id'];
				$this->ACLTable[$aclId]->addGroupEntry($record);
			}
		}
		else {
			$this->debug('ACLManager::loadACL, error : unable to load groups for ACL(id='.$csv.')', 3);
			return false;
		}
		
		if ($res = $db->getRecord(ACL_USERS_TABLE, -1, $cond)) {
			while ($record = $db->fetchArray($res)) {
				$aclId = (int)$record['acl_id'];
				$this->ACLTable[$aclId]->addUserEntry($record);
			}
		}
		else {
			$this->debug('ACLManager::loadACL, error : unable to load users for ACL(id='.$csv.')', 3);
			return false;
		}
		
		$this->debug('ACLManager::loadACL, success : ACL(id='.$csv.') loaded', 9);
		
		return true;
	}
	
	/**
	 * ACL Exists
	 *
	 * Checks if ACL is defined into manager
	 *
	 * @param  integer $aclId ACL Id
	 * @return boolean
	 */
	function ACLExists($aclId) {
		return isset($this->ACLTable[$aclId]);
	}
	
	/**
	 * Deletes ACL
	 *
	 * Remove ACL from database and manager
	 *
	 * @author vinadelmar <vinadelmar@users.sourceforge.net>
	 * @param  integer $aclId
	 * @return boolean
	 */
	function deleteACL($aclId) {
		if (!$this->ACLExists($aclId)) {
			return false;
		}
		$db = &$this->getDBManipulator();
		
		if (!$db->delRecord(ACL_TABLE, $aclId) || !$db->getAffectedRows()) {
			$this->debug('ACLManager::deleteACL, error : unable to delete ACL(id='.$aclId.')', 3);
			return false;
		}
		
		$res = $db->delRecord(ACL_GROUPS_TABLE, $aclId, '', 'acl_id');
		
		if ($res && $db->getAffectedRows()) {
			$this->debug('ACLManager::deleteACL, success : users deleted from ACL(id='.$aclId.')', 9);
		}
		
		$res = $db->delRecord(ACL_USERS_TABLE, $aclId, '', 'acl_id');
		
		if ($res && $db->getAffectedRows()) {
			$this->debug('ACLManager::deleteACL, success : groups deleted from ACL(id='.$aclId.')', 9);
		}
		
		return true;
	}
	
	/**
	 * Parent gives its ACL to its Children
	 *
	 * @param  integer $aclId
	 * @return integer acl id or 0
	 * @deprecated
	 */
	function restoreInheritance($aclId) {
		$child = &$this->getACL($aclId);
		
		if (is_null($child)) {
			return 0;
		}
		
		$childParentId = $child->getAclParentId();
		
		if (!$childParentId) {
			return 0;
		}
		
		$parent = &$this->getACL($childParentId);
		
		if (is_null($parent)) {
			return 0;
		}
		
		return $parent->getAclId();
	}
	
	/**
	 * Stores ACL
	 *
	 * @param  integer $aclId
	 * @return integer
	 */
	function storeACL($aclId) {
		$this->debug('ACLManager::storeACL, status : method invoked', 10);
		$db = &$this->getDBManipulator();

		// new acl
		if (!$aclId) {
			$res = $db->addRecord(ACL_TABLE, array());
			
			if (!$res || !$db->getAffectedRows()) {
				return 0;
			}
			
			$newId = $db->getLastId();
			
			$newAcl = &$this->getACL(0);
			$newAcl->setAclId($newId);
			
			foreach ($newAcl->getGroupsAclEntryTable() as $groupId => $aclEntry) {
				if (!$aclEntry->isEmpty()) {
					$record = $aclEntry->getValues();
					$record['acl_id'] = $newId;
					$record['group_id'] = $groupId;
					
					$res = $db->addRecord(ACL_GROUPS_TABLE, $record);
					
					if (!$res || !$db->getAffectedRows()) {
						return 0;
					}
				}
			}
			
			foreach ($newAcl->getUsersAclEntryTable() as $userId => $entry) {
				if (!$entry->isEmpty()) {
					$fieldsArray = $entry->getValues();
					$fieldsArray['acl_id'] = $newId;
					$fieldsArray['user_id'] = $userId;
					
					$result = $this->dbManipulator->queryInsert(ACL_USERS_TABLE, $fieldsArray);
					
					if (!$result || !$this->dbManipulator->getAffectedRows()) {
						return 0;
					}
				}
			}
			
			$this->setACL($newAcl);
			
			unset($newAcl);
			return $newId;
		} //if (!$aclId)
		
		$oldAcl = $this->ACLTable[$aclId];
		$acl = $this->ACLTable[0];
		$acl->setAclId($aclId);
		
		foreach ($acl->getGroupsAclEntryTable() as $groupId => $entry) {
			if ($oldAcl->groupAclEntryExists($groupId)) {
				if ($entry->isEmpty()) {
					$result = $this->dbManipulator->queryDelete(ACL_GROUPS_TABLE, $aclId, '`group_id`='.$groupId, 'acl_id');
					
					if (!$result || !$this->dbManipulator->getAffectedRows()) {
						return 0;
					}
					
					$acl->removeGroupAclEntry($groupId);
				} else {
					$oldEntry = &$oldAcl->getGroupAclEntry($groupId);
					
					if (!$entry->compareRights($oldEntry)) {
						$fieldsArray = $entry->getValues();
						$fieldsArray['acl_id'] = $aclId;
						$fieldsArray['group_id'] = $groupId;
						
						$result = $this->dbManipulator->setRecord(ACL_GROUPS_TABLE, $aclId, $fieldsArray, '`group_id`='.$groupId, 'acl_id');
						
						if (!$result || !$this->dbManipulator->getAffectedRows()) {
							$this->debug('ACLManager::storeACL, error : could not update entry', 3);
							return 0;
						}
					}
				}
			}
      else {
				$fieldsArray = $entry->getValues();
				$fieldsArray['acl_id'] = $aclId;
				$fieldsArray['group_id'] = $groupId;
				
				$result = $this->dbManipulator->queryInsert(ACL_GROUPS_TABLE, $fieldsArray);
				
				if (!$result || !$this->dbManipulator->getAffectedRows()) {
					return 0;
				}
			}
		}
		
		foreach ($acl->getUsersAclEntryTable() as $userId => $entry) {
			if ($oldAcl->userAclEntryExists($userId)) {
				if ($entry->isEmpty()) {
					$result = $this->dbManipulator->queryDelete(ACL_USERS_TABLE, $aclId, '`user_id`='.$userId, 'acl_id');
					
					if (!$result || !$this->dbManipulator->getAffectedRows()) {
						return 0;
					}
					
					$acl->removeUserAclEntry($userId);
				} else {
					$oldEntry = &$oldAcl->getUserAclEntry($userId);
					
					if (!$entry->compareRights($oldEntry)) {
						$fieldsArray = $entry->getValues();
						$fieldsArray['acl_id'] = $aclId;
						$fieldsArray['user_id'] = $userId;
						
						$result = $this->dbManipulator->queryUpdate(ACL_USERS_TABLE, $fieldsArray, $aclId, 'acl_id', '`user_id`='.$userId);
						
						if (!$result || !$this->dbManipulator->getAffectedRows()) {
							return 0;
						}
					}
				}
			}
      else {
				$fieldsArray = $entry->getValues();
				$fieldsArray['acl_id'] = $aclId;
				$fieldsArray['user_id'] = $userId;
				
				$result = $this->dbManipulator->queryInsert(ACL_USERS_TABLE, $fieldsArray);
				
				if (!$result || !$this->dbManipulator->getAffectedRows()) {
					return 0;
				}
			}
		}
		
		$this->ACLTable[$aclId] = $acl;
		
		unset($acl);
		
		return $aclId;
	}
    

	/**
	 * Removes ACL
	 *
	 * Removes ACL from manager
	 *
	 * @param  integer $id ACL Id
	 * @return boolean
	 */
  function removeACL($id) {
		if (!$this->ACLExists($id)) {
			$this->debug('ACLManager::removeACL, error : '
				. 'ACL(id='.$id.') unknown', 3);
			return false;
		}
		unset($this->ACLTable[$id]);
		$this->debug('ACLManager::removeACL, success : '
			. 'ACL(id='.$id.') removed', 9);
		return true;
  }
	
	/**
	 * Gets ACL
	 * 
	 * Checks if ACL is not already defined into manager, if not try to load it
	 *
 	 * @param  integer $id ACL Id
 	 * @param  boolean $loadList Loads ACLEntry objects too
 	 * @return object null if error
	 */	
  function &getACL($aclId, $loadList=true, $reload=false) {
		if ($reload || !$this->ACLExists($aclId)) {
			if (!$this->loadACL($aclId, $loadList)) {
				$this->debug('ACLManager::getACL, error : unable to get ACL #'.$aclId, 3);
				/*
				$acl = &$this->getNewACL();
				return $acl;
				*/
				return null;
			}
		}
		$this->debug('ACLManager::getACL, success : got ACL #'.$aclId, 9);
		return $this->ACLTable[$aclId];
  }

	/**
	 * Saves ACL
	 *
	 * Saves ACL into manager
	 *
	 * @param  object $acl ACL
	 * @return boolean
	 */
  function setACL(&$acl) {
		$aclId = $acl->getAclId();
		if (!$this->ACLExists($aclId)) {
			$this->ACLTable[$aclId] = &$acl;
		}
		return true;
	}
	
	function &getNewACL() {
		$this->ACLTable[0] = &new ACL();
		return $this->ACLTable[0];
	}
	
	function loadCSV()
	{
	}

	// deprecated
	function toACL($values, $empty_entries=true) {
		$entryString = '`^[gu]{1}_\d+:[01]{1},[01]{1},[01]{1},[01]{1},[01]{1}$`';
		
		$acl = new ACL();
		
		foreach ($values as $key => $csv) {
			if (!preg_match($entryString, $key.':'.$csv)) {
				continue;
			}
			
			$rights = explode(',', $csv);
			$view    = (int)$rights[0];
			$edit    = (int)$rights[1];
			$publish = (int)$rights[2];
			$remove  = (int)$rights[3];
			$admin   = (int)$rights[4];
			
			if (!$empty_entries && !($view || $edit || $publish || $remove || $admin)) {
				continue;
			}
			
			// $key: 'g_17', 'u_78', ... 'g' or 'u' only
			if ($key[0] == 'g') {
				$groupId = (int)substr($key, 2);
				$acl->addGroupAclEntry($groupId, $view, $edit, $publish, $remove, $admin);
			}
			else {
				$userId = (int )substr($key, 2);
				$acl->addUserAclEntry($userId, $view, $edit, $publish, $remove, $admin);
			}
		}
		
		return $acl;
	}
	/**#@-*/	
} 

?>
