<?php
/*
 * This file is part of the Easy-CMS project.
 * Easy-CMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Easy-CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Easy-CMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @package Core
 */

/**
 */
include_once('classes/core/CoreObject.class.php');

/**
 * Abstract class for managing values contained in Arrays.
 *
 * @access  private
 * @author	Cedric Francoys <cedric.francoys@easy-cms.org>
 * @version	ValuesArray.class, Thu Dec 02 19:16:04 CET 2004
 * @package	Core
 */
class ValuesArray extends CoreObject
{
	/**#@+
	 * @access private
	 */
	/**
	 * @var array
	 */
	var $originalValuesTable;

	/**
	 * @var array
	 */
	var $valuesTable;

	/**
	 * @var string
	 */
    var $filter;
	/**#@-*/

	/**
	 * Class constructor.
	 *
	 * Initialize the ValuesArray object using a reference to an array.
	 *
	 * @access   public
	 * @abstract
 	 * @param    array    The array we want the values to be manipulated.
 	 * @param    string   A filter to make a distinction between variables of different owners but contained in the same array.
 	 * @param    bool     Tells if the data contained in the array are serialized or not.
 	 * @return   void
	 */
    function ValuesArray(&$array, $filter='', $serialized=false)
	{
    	$this->originalValuesTable = &$array;
    	$this->setFilter($filter);
    	if (isset($array) && $array != null) {
	    	foreach ($array as $key => $value) {
	    		if ($serialized && $value != null) {
	    			$value = &unserialize($value);
					//?continue;
				}
				$this->valuesTable[$key] = &serialize($value);
			}
    	}
    }

	function &getValuesTable() {
		return $this->originalValuesTable;
	}

    function &getVar($name, $defaultValue = null)
	{
    	$fname = $this->applyFilter($name); // filtered name
    	if (!$this->checkVar($fname)) {
   	        return $defaultValue;
		}
		return unserialize($this->valuesTable[$fname]);
    }

	/**
	 * @param  string $name
	 * @return mixed
	 */
    function applyFilter($name)
	{
		return $this->filter.$name;
	}

	/**
	 * @param  string
	 * @return mixed
	 */
    function checkVar($fname)
	{
        return (isset($this->valuesTable[$fname]) && $this->valuesTable[$fname] != null);
    }

    function &getFilter()
    {
    	return $this->filter;
    }

    function setFilter($filter)
    {
    	$this->filter = &$filter;
    }

    function storeVar($name, &$value)
    {
		$this->debug("ValuesArray.storeVar invoked, name: '$name', value: '$value'.", 10);

		$fname = $this->applyFilter($name);

		$this->debug("ValuesArray.storeVar, filtered name: '$fname'.", 10);

		if(isset($this->valuesTable[$fname]) && $this->valuesTable[$fname] != null){
			$this->debug("ValuesArray.storeVar, warning : var '$name' already defined (var overwritten).", 4);
		}
		$this->valuesTable[$fname] = &serialize($value);
    }

    function removeVar($name)
    {
    	$fname = $this->applyFilter($name);
		$this->valuesTable[$fname] = null;
    }

    function removeAllVars()
    {
		foreach($this->valuesTable as $key => $value){
			$this->valuesTable[$key] = null;
		}
    }

	/**
	 * Copies all the values to a specified array
	 *
	 * @param  string The name of the value to be copied
	 * @param  array The array where the value is to be copied to
 	 * @return   void
	 */
    function copyTo(&$array)
    {
    	foreach($this->valuesTable as $key => $value){
			$array[$key] = $value;
		}
	}

	/**
	 * Copies a value to a specified array
 	 *
	 * @param  string The name of the value to be copied
	 * @param  array The array where the value is to be copied to
 	 * @return   void
	 */
	function copyVarTo($name, &$array)
	{
    	$fname = $this->applyFilter($name);
    	if(!$this->checkVar($fname)){
			$this->debug("ValuesArray.copyVarTo, warning : var '$name' not defined.", 4);
			return;
		}
		$array[$fname] = &$this->valuesTable[$fname];
    }
}

?>