<?php
/*
 * This file is part of the Easy-CMS project.
 * Easy-CMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Easy-CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Easy-CMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
/**
 * @package    Core
 * @subpackage Templates
 */

/**
 *
 */
include_once('config.inc.php');
include_once('classes/core/DBAccessorObject.class.php');
include_once('classes/core/templates/Template.class.php');

/**
 * TemplatesManager Class
 *
 * Manage instances of Template objects.
 *
 * @access     public
 * @author	   Cedric Francoys <cedric.francoys@easy-cms.org>
 * @version    TemplatesManager.class,v 0.1 
 * @package	   Core
 * @subpackage Templates
 */
class TemplatesManager extends DBAccessorObject {
	/**#@+
	 * @access private
	 */
	/**
	 * @var array
	 */
    var $templatesTable;
	/**#@-*/
	
	/**
	 * Constructor
	 *
	 * @param object $dbManipulator
	 */
	function TemplatesManager(&$dbManipulator) {
		parent::DBAccessorObject($dbManipulator);		
	}
	
	/**
	 * Template Exists
	 *
	 * Checks if Template of given type and id is defined into manager
	 *
	 * @param  string  $typeName Template Type
	 * @param  integer $id Template Id
	 * @return boolean
	 */
	function templateExists($typeName, $id){
		return isset($this->templatesTable[$typeName][$id]);
	}
	
	/**
	 * Loads Template
	 *
	 * @param  string $typeName Template Type
	 * @param  integer $templateId Template Id
	 * @return boolean
	 */
  function loadTemplate($typeName, $templateId) {
		$templatesIdList = (array)$templateId;
		$csv = implode(',', $templatesIdList);
		$db = &$this->getDBManipulator();
		$where  = '`type`='.$db->quoteSmart($typeName);
		$where .= ' and `id` in ('.$csv.')';
		$res = $db->getRecord(TEMPLATES_TABLE, -1, $where);
		if (!$res || !$db->getNumRows()) {
			$dmsg  = 'TemplatesManager::loadTemplate, error : unable to ';
			$dmsg .= 'load Template(id='.$csv.' type_name="'.$typeName.'")';
			$this->debug($dmsg, 3);
			return false;
		}
		while ($record = $db->fetchArray($res)) {
			$id   = (int)$record['id'];
			$name = $record['name'];
			$xsl  = $record['xsl'];
			$this->templatesTable[$typeName][$id] = &new Template($id, $typeName, $name, $xsl);
		}
		$dmsg  = 'TemplatesManager::loadTemplate, success : ';
		$dmsg .= 'Template(id='.$csv.' type_name="'.$typeName.'") loaded';
		$this->debug($dmsg, 9);
		return true;
  } 

	/**
	 * Stores Template
	 *
	 * @param  integer $templateId Template Id
	 * @return integer Template Id or 0 if error
	 */
  function storeTemplate($templateId){
		if (!$this->templateExists($templateId)) {
			$dmsg  = 'TemplatesManager::storeTemplate, error : ';
			$dmsg .= 'Template(id='.$templateId.') unknown';
			$this->debug($dmsg, 3);
			return 0;
		}
		$db = &$this->getDBManipulator();
		$template = &$this->getTemplate($typeName, $templateId);
		$record = $template->toArray();
		if (!$templateId) {
			$res = $db->addRecord(TEMPLATES_TABLE, $record);
			if (!$res || !$db->getAffectedRows()) {
				return 0;
			}
			$newId = $db->getLastId();
			$template->setTemplateId($newId);
			$templateId = $newId;
		} else {
			if (!$db->setRecord(TEMPLATES_TABLE, $templateId, $record)) {
				return 0;
			}
		}
		return $templateId;
  } 
	
	/**
	 * Removes Template
	 *
	 * @param  integer $templateId
	 * @return boolean
	 */
  function removeTemplate($templateId){
		$db = &$this->getDBManipulator();
		$record = array();
		$record['deleted'] = 1;
		if (!$db->setRecord(TEMPLATES_TABLE, $templateId, $record)) {
			return false;
		}
		return true;
  } 
	
  /**
	 * Gets Template
	 *
	 * @param  string $typeName Template Type
	 * @param  integer $id Template Id
	 * @return object
	 */
  function &getTemplate($typeName, $id) {  
    if (!$this->templateExists($typeName, $id)) {
			if (!$this->loadTemplate($typeName, $id)) {
				$this->debug('TemplatesManager::getTemplate, error : unable to get Template(id='.$id.' type_name="'.$typeName.'")', 3);
    	  return null;
			}
		}
		$this->debug('TemplatesManager::getTemplate, status : got Template(id='.$id.' type_name="'.$typeName.'")', 10);
    return $this->templatesTable[$typeName][$id];
  } 

	/**
	 * Gets Template Id
	 *
	 * @param  string $templateName Template Name
	 * @return integer
	 */
  function getTemplateId($templateName){
		$db = &$this->getDBManipulator();
		$where = '`name`='.$db->quoteSmart($templateName);
		$res = $db->getRecord(TEMPLATES_TABLE, -1, $where);
		if (!$res || !$db->getNumRows()) {
			return 0;
		}
		$record = $db->fetchArray($res);
		$templateId = (int)$record['id'];
		return $templateId;
  }
	
	/**
	 * @deprecated use {@link templateExists}
	 * @todo approve deprecation
	 */
	/*
	function checkTemplate($typeName, $id){
    return $this->templateExists($typeName, $id);
  }
	*/
} 

?>
