<?php
/*
 * This file is part of the Easy-CMS project.
 * Easy-CMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Easy-CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Easy-CMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
/**
 * @package    Core
 * @subpackage Types
 */

/**
 */
include_once('classes/core/DBAccessorObject.class.php');
include_once('classes/core/types/Type.class.php');
include_once('classes/core/DBManipulator.class.php');
include_once('classes/types/Bool.class.php');
include_once('classes/types/Color.class.php');
include_once('classes/types/Date.class.php');
include_once('classes/types/Document.class.php');
include_once('classes/types/Image.class.php');
include_once('classes/types/Integer.class.php');
include_once('classes/types/Float.class.php');
include_once('classes/types/Reference.class.php');
include_once('classes/types/String.class.php');
include_once('classes/types/ShortText.class.php');
include_once('classes/types/Link.class.php');
include_once('classes/types/Text.class.php');
include_once('classes/types/Time.class.php');


/**
 * Manages types and instantiates objects parts of content.
 *
 * @author	   Cedric Francoys <cedric.francoys@easy-cms.org>
 * @version	   ShortText.class, Sun Nov 28 13:26:21 CET 2004
 * @package    Core
 * @subpackage Types
 */
class TypesManager extends DBAccessorObject {
	/**
	 * Type Objects Array
	 *
	 * @access private
	 * @var array
	 */
	var $typesTable;

	/**
	 * Constructor
	 *
	 * @access public
	 * @param object  $dbManipulator
	 */
	function TypesManager(&$dbManipulator) {
		parent::DBAccessorObject($dbManipulator);
	}
	
	/**
	 * @return array
	 */
	function &getTypesTable() {
		return $this->typesTable;
	}

	/**
	 * Loads Type
	 *
	 * loadType first checks if an instance already exists. if not it will 
	 * try to instanciate the type if its corresponding table exists in the 
	 * database.
	 *
	 * @access private
	 * @param  string  $name type's name
	 * @return boolean
	 */
	function loadType($name) {
		if (!$this->dbManipulator->typeExists($name)) {
			return false;
		}
		
		$this->typesTable[$name] = &new Type($name, $this->dbManipulator);
		
		return true;
	}

	/**
	 * Check if type exists
	 *
	 * @access private
	 * @param  string $name
	 * @return boolean
	 */
	function typeExists($name) {
		return isset($this->typesTable[$name]);
	}

    /**
	 * Gets Type
	 *
	 * Returns user defined Type object
	 *
	 * @access public
 	 * @param  string $name
 	 * @return object 
	 */
	function &getType($name) {
		if (!$this->typeExists($name)) {
			if (!$this->loadType($name)) {
				return null;
			}
		}
		return $this->typesTable[$name];
	}
	
	/**
	 * Creates Easy-CMS Simple Type
	 *
	 * @access public
	 * @param  integer $type
	 * @param  mixed $value
	 * @return object returns null if unknown type
	 */
	function createTypeObject($type, &$value) {
		switch ($type) {
		case BOOL:
			$obj = &new Bool($value);
			break;
			
		case INTEGER:
			$obj = &new Integer($value);
			break;
			
		case FLOAT:
			$obj = &new Float($value);
			break;
			
		case DATE:
			$obj = &new Date($value);
			break;
			
		case TIME:
			$obj = &new Time($value);
			break;
			
		case COLOR:
			$obj = &new Color($value);
			break;
			
		case REFERENCE:
			$obj = &new Reference($value);
			break;		
			
		case STRING:
			$obj = &new String($value);
			break;
			
		case SHORTTEXT:
			$obj = &new ShortText($value);
			break;
			
		case TEXT:
			$obj = &new Text($value);
			break;
			
		case LINK:
			$obj = &new Link($value);
			break;
			
		case DOCUMENT:
			$obj = &new Document($value);
			break;
			
		case IMAGE:
			$obj = &new Image($value);
			break;
			
		default:
			$obj = null;
		}
		
		return $obj;
	}
	
	/**
	 * Checks if instance of specified type exist
	 *
	 * @param  string $name type's name
	 * @return boolean
	 * @deprecated due to naming convention
	 * @todo approve deprecation
	 */
    function checkType($name) 
	{
        return $this->typeExists($name);
    }
	
	/**
	 * @deprecated
	 * @todo approve deprecation
	 */
	function countDBTypes()
	{
		return $this->dbManipulator->countTypeTables();
	}
	
	/**
	 * @deprecated
	 * @todo approve deprecation
	 */
    function getTypesCount() {
        if (isset($this->typesTable))
            return count($this->typesTable);
        return 0;
    }
	
	/**
	 * @deprecated
	 * @todo approve deprecation
	 */
    function storeType($name) 
	{
        //$this->typesTable[$name] = new Type($name, $this->dbConnection);
    } 
} 

?>